# Supports Organizer.rb
=begin rdoc

= dropGC.rb
Copyright 2012 by Rick Wilson - All Rights Reserved

== Disclaimer
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

== License
This software is distributed under the Smustard End User License Agreement
http://www.smustard.com/eula

== Information
Author:: Rick Wilson
Organization:: Smustard
Name:: dropGC.rb
Version:: 1.0.1
SU Version:: 4.0
Date:: 2013-01-03
Description:: drop selected groups/components to the first face below each instance

Usage::
* 1:: Install into the plugins directory or into the Plugins/examples directory and manually load from
	   the ruby console "load 'examples/dropGC.rb'" 
* 2:: Plugins>DropGC or right-click>DropGC activates the tool

History::
* 1.0.1:: 2013-01-03
	* fixed bug that caused the menu entry item to not function
* 1.0.0:: 2012-02-20
	* first version

ToDoList::
	* add options & settings for: use hidden geometry (SU8m1 and up)
	* add options & settings for: drop by set amount
	* add options & settings for: drop to Z
	* add options & settings for: drop to selected
	* add options & settings for: drop to selected (virtual extended plane)
	* add options & settings for: drop to intersection plus additional offset up or down
	* add options & settings for: keep G/C above face below
	* add options & settings for: match G/C XY plane to intersected plane below

=end


require "sketchup.rb"

module Smustard

class DropGC
	
	@@menucreated = false
	#@@path = Sketchup.find_support_file("Plugins")+"/"
	@@path = File.dirname(__FILE__)+"/"
	
	attr_accessor :wysiwig, :last_time
	
	def initialize()
		
		@model = Sketchup.active_model
		@selection = @model.selection
		@wysiwig = true
		begin
			Sketchup.active_model.raytest([[0,0,0],[0,0,-1]],@wysiwig)
		rescue
			@wysiwig = false
		end
		@last_time = 0
	end
	
	
	def reinit(wysiwig=true)
		
		@model = Sketchup.active_model
		@selection = @model.selection
		@wysiwig = wysiwig
		
	end
	
	
	def drop(sel=@selection,wysiwig=true)
		reinit() if Sketchup.active_model != @model
		return nil unless sel.kind_of?(Array) || sel.kind_of?(Sketchup::Selection)
		down = [0,0,-1]
		qty = sel.count
		dropped = 0
		(Sketchup.version.split('.')[0].to_i)>6 ? @model.start_operation("DropGC",true) : @model.start_operation("DropGC")
		
		start_time = Time.now()
		counting = false
		counting = true if sel.length > 10
		
		sel.each_with_index do |instance,i|
			
			(Sketchup.set_status_text("Processing: #{((i+1)*100)/qty}%") if  (i+1).modulo(qty/10)==0) if (counting && qty>0)
			
			next unless instance.kind_of?(Sketchup::ComponentInstance) || instance.kind_of?(Sketchup::Group)
			origin = instance.transformation.origin
			
			v,m,b = Sketchup.version.split('.')
			
			if v.to_i > 7 && b.to_i > 4809
				next unless raytest_result = @model.raytest([origin,down],wysiwig)
			else
				next unless raytest_result = @model.raytest([origin,down])
			end
			
			instance.transform!(Geom::Transformation.translation(raytest_result[0]-origin))
			dropped += 1
			
		end
		
		end_time = Time.now()
		
		@last_time = end_time - start_time
		
		@model.commit_operation()
		
		Sketchup.set_status_text("Dropped #{dropped} items in #{@last_time} seconds")
		
		#UI.messagebox("Dropped #{dropped} items in #{@last_time} seconds")
		
	end
	
	
	def settings()
		#	This code will allow the user to change the DropGC settings.  Settings could include:
		#		* use hidden geometry (SketchUp 8m1 and higher)
		#		* drop by set amount
		#		* drop to Z
		#		* drop to selected
		#		* drop to selected (virtual extended plane)
		#		* drop to intersection plus additional offset up or down
		#		* keep G/C above face below
		#		* match G/C XY plane to intersected plane below
	end
	
	
	def create_menu()
		
		begin
		menunames = IO.readlines(File.dirname(__FILE__)+"/menus/"+File.basename(__FILE__, ".rb")+".menu").collect{|x| x.chop}
		
		rescue
		menunames = ["DropGC","DropGC Settings"]
		
		end

		cmd_dropgc = UI::Command.new(menunames[0]){ $smustardDropGC.drop() }
		cmd_dropgc.tooltip = "Drop Groups and/or Components to geometry below"
		cmd_dropgc.large_icon = "#{@@path}smustard/dropGClarge.png"
		cmd_dropgc.small_icon = "#{@@path}smustard/dropGCsmall.png"
		$smustard_toolbar.add_item(cmd_dropgc) if $smustard_toolbar
		
		#cmd_dropgcsettings = UI::Command.new(menunames[1]){$smustardDropGC.settings()}
		
		
		unless @@menucreated
		
			$submenu ? (organizer = $submenu) : (organizer = UI.menu("Plugins"))
			organizer.add_item(cmd_dropgc)
			#organizer.add_item(menunames[1]) { cmd_dropgcsettings }
			
			UI.add_context_menu_handler do |menu|
				menu.add_item(cmd_dropgc)
			end
		
			@@menucreated = true
		
		end
		
		
	end
	
	
end #class DropGC


end #module Smustard

unless file_loaded?(__FILE__)
	file_loaded(__FILE__)

	$smustardDropGC = Smustard::DropGC.new
	$smustardDropGC.create_menu()
	
end